//
//  AppDelegate.m
//
//  Created by Cardinal Development on 8/15/14.
//  Copyright (c) 2014 Cardinal Scale Manufacturing Company. All rights reserved.
//

#import "AppDelegate.h"

@interface AppDelegate()

@property (nonatomic) BOOL outputStreamConnected;
@property (nonatomic) NSMutableString *rcv_buf;
@property (nonatomic) NSInteger rcvMode;
@property (nonatomic) NSInputStream *inputStream;
@property (nonatomic) NSOutputStream *outputStream;

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.isConnected = NO;
    
    self.cmdQueue = [[NSMutableArray alloc] init];
    
    // Load user defaults
    self.defaults = [NSUserDefaults standardUserDefaults];
    
    self.curIPAddress = [self.defaults objectForKey:@"ip_address"];
    self.curPortNumber = [self.defaults objectForKey:@"port_number"];
    
    if (self.curIPAddress == nil) {
        self.curIPAddress = @"192.168.0.100";
        [self.defaults setObject:self.curIPAddress forKey:@"ip_address"];
        [self.defaults synchronize];
    }
    
    if (self.curPortNumber == nil) {
        self.curPortNumber = @"10001";
        [self.defaults setObject:self.curPortNumber forKey:@"port_number"];
        [self.defaults synchronize];
    }
    
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

#pragma mark - NSStreamDelegate Methods

- (void)stream:(NSStream *)theStream handleEvent:(NSStreamEvent)eventCode
{
    switch (eventCode)
    {
        case NSStreamEventOpenCompleted:
            NSLog(@"Stream opened");
            break;
        case NSStreamEventHasBytesAvailable:
            if (theStream == self.inputStream)
            {
                uint8_t buffer[1024];
                NSInteger len;
                
                len = [self.inputStream read:buffer maxLength:(NSUInteger)sizeof(buffer)];
                
                for (int i = 0; i < len; ++i)
                {
                    if (self.rcvMode == 0)
                    {
                        if (buffer[i] == '\n')
                        {
                            [self.rcv_buf setString:@""];
                            self.rcvMode = 1;
                        }
                    }
                    else
                    {
                        switch (self.rcvMode)
                        {
                            case 1:
                                if (buffer[i] == '\r')
                                {
                                    // Parse the weight string
                                    if (self.rcv_buf.length == 18) {
                                        NSString *weight = [[self.rcv_buf substringWithRange:NSMakeRange(5, 10)] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                                        NSString *units = [self.rcv_buf substringWithRange:NSMakeRange(15, 3)];
                                        NSString *mode = [NSString stringWithFormat:@"%c", [self.rcv_buf characterAtIndex:2]];
                                        NSString *status = @"";
                                        
                                        if ([mode isEqualToString:@"G"]) {
                                            mode = @"GROSS";
                                        } else if ([mode isEqualToString:@"N"]) {
                                            mode = @"NET";
                                        } else if ([mode isEqualToString:@"T"]) {
                                            mode = @"TARE";
                                        }
                                        
                                        if ([self.rcv_buf characterAtIndex:3] == 'M') {
                                            status = @"MOTION";
                                        } else if ([self.rcv_buf characterAtIndex:0] == 'Z') {
                                            status = @"CENTER OF ZERO";
                                        } else if ([self.rcv_buf characterAtIndex:0] == 'U') {
                                            status = @"BELOW ZERO";
                                        } else if ([self.rcv_buf characterAtIndex:0] == 'O') {
                                            status = @"OVER CAPACITY";
                                        }
                                        
                                        // We now have the weight, units, and status and can now do something with them
                                    }
                                    
                                    self.rcvMode = 0;
                                }
                                else
                                {
                                    [self.rcv_buf appendFormat:@"%c", buffer[i]];
                                    
                                    if (self.rcv_buf.length > 2048)
                                    {
                                        // Something went wrong!
                                        self.rcvMode = 0;
                                    }
                                }
                                break;
                            default:
                                self.rcvMode = 0;
                        }
                    }
                }
            }
            break;
        case NSStreamEventErrorOccurred:
            NSLog(@"NSStream error occurred");
            if (theStream == self.outputStream)
            {
                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"ERROR"
                                                                    message:@"Could not connect to host"
                                                                   delegate:nil
                                                          cancelButtonTitle:@"OK"
                                                          otherButtonTitles:nil];
                [alertView show];
            }
            
            [self.setupView.swConnect setOn:NO];
            break;
        case NSStreamEventEndEncountered:
            [theStream close];
            [theStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [self.setupView.swConnect setOn:NO];
            break;
        case NSStreamEventHasSpaceAvailable:
            if (self.outputStreamConnected == false)
            {
                NSLog(@"Output stream connected");
                self.outputStreamConnected = true;
                
                // Start weight request timer
                [NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(onTick:) userInfo:nil repeats:true];
            }
            break;
        default:
            NSLog(@"Unknown event!");
    }
}

- (void)onTick:(NSTimer *) timer
{
    NSString *commandToSend = nil;
    
    if ([self.cmdQueue count]) {
        commandToSend = (NSString *)[self.cmdQueue lastObject];
        [self.cmdQueue removeLastObject];
    }
    
    if (self.outputStreamConnected){
        if (commandToSend) {
            NSData *data = [[NSData alloc] initWithData:[commandToSend dataUsingEncoding:NSASCIIStringEncoding]];
            [self.outputStream write:[data bytes] maxLength:[data length]];
        } else {
            NSString *wtRequest = @"\nW\r";
            NSData *data = [[NSData alloc] initWithData:[wtRequest dataUsingEncoding:NSASCIIStringEncoding]];
            [self.outputStream write:[data bytes] maxLength:[data length]];
        }
    } else {
        [timer invalidate];
    }
}

- (void)connect
{
    // Init networking
    self.outputStreamConnected = false;
    self.rcv_buf = [[NSMutableString alloc] init];
    self.rcvMode = 0;
    
    CFReadStreamRef readStream;
    CFWriteStreamRef writeStream;
    
    CFStreamCreatePairWithSocketToHost(NULL, (__bridge CFStringRef)self.curIPAddress, (UInt32)[self.curPortNumber integerValue], &readStream, &writeStream);
    
    self.inputStream = (__bridge_transfer NSInputStream *)readStream;
    self.outputStream = (__bridge_transfer NSOutputStream *)writeStream;
    
    [self.inputStream setDelegate:self];
    [self.outputStream setDelegate:self];
    
    [self.inputStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [self.outputStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    
    [self.inputStream open];
    [self.outputStream open];
}

- (void)disconnect
{
    [self.inputStream close];
    [self.inputStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    
    [self.outputStream close];
    [self.outputStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    self.outputStreamConnected = false;
    
    [self.setupView.swConnect setOn:NO];
}

@end
